#region Using directives

using System;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using System.Collections;

#endregion

namespace BitmapHandler
{
    public unsafe class BitmapHandler : IDisposable
    {
        private Bitmap bmp; // the forms bitmap
        private BitmapData bmpdata;
        byte* bptr;
        int stride;
        private Random rnd;
        private int btop; // top y of drawn area
        private int bleft; // top x of drawn area
        private int bbottom; // bottom y of drawn area
        private int bright; // bottom x of drawn area

        //colour testing
        private Color[] Cols;
        private Hashtable hashcols;
        private int coltest;

       // constructor
        public BitmapHandler(Bitmap bm, int top, int left, int bottom, int right)
        {
            bmp = bm;
            rnd = new Random();
            btop = top;
            bleft = left;
            bbottom = bottom;
            bright = right;

            // colour testing
            Cols = new Color[5];
            Cols[0] = Color.Black;
            Cols[1] = Color.Red;
            Cols[2] = Color.Green;
            Cols[3] = Color.Blue;
            Cols[4] = Color.White;
            hashcols = new Hashtable();
            hashcols.Add(Cols[0].ToArgb(), 0);
            hashcols.Add(Cols[1].ToArgb(), 1);
            hashcols.Add(Cols[2].ToArgb(), 2);
            hashcols.Add(Cols[3].ToArgb(), 3);
            hashcols.Add(Cols[4].ToArgb(), 4);
        }

        // public properties
        public Bitmap ControlRef
        { get { return bmp; } }

        public Bitmap Value
        { get { return bmp;}}

        // public methods
       private void setcol(int x,int y,int col)
        {
            byte* pixel = bptr + y*stride + x*3;
            *pixel++ = (byte)col;
            *pixel++ = (byte)(col >> 8);
            *pixel = (byte) (col >> 16);
        }

        private int getcol(int x, int y)
        {
            byte* pixel = bptr + y*stride + x*3+2;
            int col = 0xff;
            col = (col << 8)+*pixel--;
            col = (col << 8)+*pixel--;
            col = (col << 8)+*pixel;
            return col;
        }

        public int timer(int[]x, int[]y, int[]col, double[] spread, double[] gravity, int n, Color bcol)
        {
            int i, nx, ny;
            bmpdata = bmp.LockBits(new Rectangle(0, 0, bmp.Width, bmp.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
            bptr = (byte*) bmpdata.Scan0;
            stride = bmpdata.Stride;
            for (i = 0; i < n; i++)
            {
                coltest =  (int)hashcols[Color.Green.ToArgb()];

                // this guards against out of range initial inputs
                // it's inefficient here and should be done at particle creation time
                // x[i] = Math.Min(Math.Max(x[i], bleft), bright);
                //y[i] = Math.Min(Math.Max(y[i], btop), bbottom);

                // calculate next position
                nx = Math.Min(Math.Max(x[i] + (int)((2 * rnd.NextDouble() - 1) * spread[i]), bleft), bright);
                ny = y[i];
                if ((gravity[i]< 0) && (rnd.NextDouble() < -gravity[i]))
                {
                    ny -=  1 ;
                }
                else if ((gravity[i]> 0) && (rnd.NextDouble() < gravity[i]))
                {
                    ny += 1;
                }
                ny = Math.Min(Math.Max(ny, btop), bbottom);

                if (getcol(nx, ny) != bcol.ToArgb())
                {
                   ny = y[i];
                }
                if(getcol(nx, ny) == bcol.ToArgb())
                {
                    //undraw old particle position
                    setcol(x[i], y[i], bcol.ToArgb());
                    x[i] = nx;
                    y[i] = ny;
                    // draw it or lose it off the bottom
                    if (ny < bbottom && ny > btop)
                    {
                        setcol(nx,ny,col[i]);
                    }
                    else
                    {
                        x[i] = x[n - 1];
                        y[i] = y[n - 1];
                        col[i] = col[n - 1];
                        spread[i] = spread[n - 1];
                        gravity[i] = gravity[n - 1];
                        n = n - 1;
                    }
                }
            }
            bmp.UnlockBits(bmpdata);
            return n;
        }

        public void Dispose()
        {
            bmp = null;
        }
    }


}
